function [CFA_Img,CFA_Pattern] = CFA_imread( fileName,mode )
% ----------------------------------------------
% MATLAB function to read CFA images using dcraw
% ----------------------------------------------
  if ispc
    % identify CFA pattern
    strCmdLine = sprintf('dcraw_MinGW.exe -i -v %s',fileName);
    [status,result] = system(strCmdLine);    
    loc = strfind(result,'Filter pattern: ')+numel('Filter pattern: '); 
    CFA_Pattern = reshape( result(loc:loc+3),[2,2] );
    
    if strcmp(mode,'Linear')   
      % extract CFA data & dump into 16-bit TIFF file
      strCmdLine = sprintf('dcraw_MinGW.exe -o 0 -r 1 1 1 1 -d -k 0 -S 65535 -4 -T %s',fileName);
      system(strCmdLine,'-echo');
      loc = strfind( fileName,'.' );
      srcFileName = strcat(fileName(1:loc(end)-1),'.TIFF');
      destFileName = strcat(fileName(1:loc(end)-1),'_CFA','.TIFF');
      delete(destFileName)
      strCmdLine = sprintf('rename %s %s',srcFileName,destFileName);
      system(strCmdLine,'-echo');
      % imread converted file
      loc = strfind( fileName,'.' );  
      CFA_Img = im2double(imread([fileName(1:loc(end)-1) '_CFA.tiff']));  
      [nPixelsY,nPixelsX] = size(CFA_Img);
      %
      CFA_Img_R = nan( nPixelsY,nPixelsX );   
      CFA_Img_G = nan( nPixelsY,nPixelsX );   
      CFA_Img_B = nan( nPixelsY,nPixelsX );   
      mtx_Ones   = true( ceil(max(nPixelsX,nPixelsY)/2) );
      %
      Mask = kron( mtx_Ones , CFA_Pattern=='R' );
      Mask = Mask(1:nPixelsY,1:nPixelsX);  
      CFA_Img_R(Mask>0) = CFA_Img(Mask>0);
      %
      Mask = kron( mtx_Ones , CFA_Pattern=='G' );
      Mask = Mask(1:nPixelsY,1:nPixelsX);  
      CFA_Img_G(Mask>0) = CFA_Img(Mask>0);
      %
      Mask = kron( mtx_Ones , CFA_Pattern=='B' );
      Mask = Mask(1:nPixelsY,1:nPixelsX);  
      CFA_Img_B(Mask>0) = CFA_Img(Mask>0);
      %
      clear Mask CFA_Img
      %
      CFA_Img = cat(3,CFA_Img_R,CFA_Img_G,CFA_Img_B);
      %
      clear CFA_Img_R CFA_Img_G CFA_Img_B
    end
    
    if strcmp(mode,'sRGB')
      % 8bit sRGB image from camera RAW image
      strCmdLine = sprintf('dcraw_MinGW.exe -w -o 1 -q 3 -T %s',fileName);
      system(strCmdLine,'-echo');
      % imread converted file
      loc = strfind( fileName,'.' );  
      CFA_Img = im2double(imread([fileName(1:loc(end)-1) '.tiff']));  
    end
    
    %
  end
  
  if ismac || isunix
    error('CFASFR is currently not supported on Unix or MAC !!')
  end
end
