function y = GaussHermite_Expansion( x,c0,c1,c2,c3,c4,sigma )
% function y = GaussHermite_Expansion( x,c0,c1,c2,c3,c4,mu,sigma )
%   y = zeros(size(x));
%   for nn=0:3
%     % linear combination of hermite polynomials
%     y = y + coeffs(nn) * polyval( HermitePoly(nn),(x-mu/sigma) )/sqrt(2^nn*sqrt(pi)*factorial(nn));
%   end
  mu = 0;
  y = c0 * polyval( HermitePoly(0),(x-mu)/sigma )/sqrt(2^0*sqrt(pi)*factorial(0)) + ...
      c1 * polyval( HermitePoly(1),(x-mu)/sigma )/sqrt(2^1*sqrt(pi)*factorial(1)) + ...
      c2 * polyval( HermitePoly(2),(x-mu)/sigma )/sqrt(2^2*sqrt(pi)*factorial(2)) + ...
      c3 * polyval( HermitePoly(3),(x-mu)/sigma )/sqrt(2^3*sqrt(pi)*factorial(3)) + ...      
      c4 * polyval( HermitePoly(4),(x-mu)/sigma )/sqrt(2^4*sqrt(pi)*factorial(4)) ;
  % gaussian window
  y = y .* exp(-((x-mu)/sigma).^2/2);
end

% HermitePoly.m by David Terr, Raytheon, 5-10-04
% ----------------------------------------------
% Given nonnegative integer n, compute the 
% Hermite polynomial H_n. Return the result as a vector whose mth
% element is the coefficient of x^(n+1-m).
% polyval(HermitePoly(n),x) evaluates H_n(x).
function hk = HermitePoly(n)
  if n==0 
    hk = 1;
  elseif n==1
    hk = [2 0];
  else
    hkm2 = zeros(1,n+1);
    hkm2(n+1) = 1;
    hkm1 = zeros(1,n+1);
    hkm1(n) = 2;
    for k=2:n
      hk = zeros(1,n+1);
      for e=n-k+1:2:n
        hk(e) = 2*(hkm1(e+1) - (k-1)*hkm2(e));
      end
      hk(n+1) = -2*(k-1)*hkm2(n+1);
      if k<n
        hkm2 = hkm1;
        hkm1 = hk;
      end
    end
  end
end
