function SFR_CFA_SlantedEdge(ESF_supersampling_factor,OPERATION,DENOISING_ALGORITHM,mode)
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Slanted Edge SFR estimation for Color Filter Array's
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

% PARAMETERS
  global CFA_struct
  
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if strcmp(mode,'VERBOSE')
  disp('User-defined ROI');
  disp(CFA_struct.ROI);    
end
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

% Identify slope & intercept of slanted edge in the green channel  
  findSlopeIntercept_SlantedEdge(mode); 
 
%   error('done')
  
% Re-center ROI around slanted edge
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if strcmp(mode,'VERBOSE')
  disp('Re-centered ROI');
end
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  nPixelsY = abs(CFA_struct.ROI(2,1) - CFA_struct.ROI(1,1)) + 1; % add 1 to get the correct num of rows
  nPixelsX = abs(CFA_struct.ROI(2,2) - CFA_struct.ROI(1,2)) + 1; % add 1 to get the correct number of cols  
  %
  Ye = round(nPixelsY/2);
  Xe = round(Ye*CFA_struct.slope+CFA_struct.intercept);
  %
  CFA_struct.ROI(1,1) = CFA_struct.ROI(1,1) + Ye - floor(nPixelsY/2) - 1;
  CFA_struct.ROI(1,2) = CFA_struct.ROI(1,2) + Xe - floor(nPixelsX/2) - 1;
  CFA_struct.ROI(2,1) = CFA_struct.ROI(1,1) + nPixelsY - 1;
  CFA_struct.ROI(2,2) = CFA_struct.ROI(1,2) + nPixelsX - 1;  
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if strcmp(mode,'VERBOSE')
  disp(CFA_struct.ROI);
end
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

% Identify slope & intercept of slanted edge in the green channel    
  findSlopeIntercept_SlantedEdge(mode); 

% -------------------------------
% Identify the super-resolved ESF 
% -------------------------------
  % The 3-point stencil [-0.5 0 0.5] is used in the ISO procedure.
  h_deriv = [-0.5,0,0.5];           % 3-point stencil...stick to antisymmetric filters of odd length in general
  
  superResolveESF(OPERATION,ESF_supersampling_factor,h_deriv(:));

  for strCh = {'Red','Green','Blue'}
    strCh = strcat(strCh{:},'_Channel');    
    
  % --------------------------------------------
  % Denoise ESF using Non-Linear Least Squares
  %   1. Replace ESF tails with sloping-sigmoids 
  %   2. Attempt cubic-spline-fit to ESF
  % --------------------------------------------
    [CFA_struct.(strCh).ESF,CFA_struct.(strCh).LSF] = ...
      DenoiseESF(CFA_struct.(strCh).ESF,h_deriv(:),DENOISING_ALGORITHM,strCh,mode);

  % ------------------------------------------------------
  % Restore Dynamic Range of each ESF to the Dynamic Range
  % of the respective color channels in the CFA image
  % ------------------------------------------------------
    CFA_struct.(strCh).ESF = ...
      adjustDR(CFA_struct.(strCh).ESF,...
               CFA_struct.(strCh).ROI_mxval,...
               CFA_struct.(strCh).ROI_mnval,1); 
             
  % -----------------------------------------------------------------  
  % Normalize LSF to have unit area
  % Area under optical PSF = unity
  % LSF in a specific direction = PSF integrated along said direction
  % ==> Area under LSF = unity
  % -----------------------------------------------------------------  
    CFA_struct.(strCh).LSF = ...
      CFA_struct.(strCh).LSF/mean(CFA_struct.(strCh).LSF(:));
  
  % -----------
  % Compute SFR  
  % -----------
    CFA_struct.(strCh).SFR = abs( fftshift(fft(CFA_struct.(strCh).LSF)) ); 
    numel_SFR = numel( CFA_struct.(strCh).SFR );                      
    CFA_struct.(strCh).SFR = CFA_struct.(strCh).SFR(round(numel_SFR/2):end);
    
    % size( CFA_struct.(strCh).SFR )
    
  % -------------------------------------------------------------------------
  % Compensate for SFR of the derivative filter used to identify LSF from ESF
  % -------------------------------------------------------------------------
    SFR_starts_at_DC = true;
    [CFA_struct.(strCh).SFR,CFA_struct.(strCh).freq] = ...
      Compensate_4_DerivativeFiltering(CFA_struct.(strCh).SFR,SFR_starts_at_DC,h_deriv);
    CFA_struct.(strCh).freq = CFA_struct.(strCh).freq *1/(CFA_struct.Pixel_Size/ESF_supersampling_factor); 
    
  % -------------------------------------
  % Fix SFR @ DC using FOTS approximation
  % -------------------------------------
    % In special cases, the SFR @ DC maybe smaller than the SFR @ the first
    % non-zero frequency. This happens due to the fact that our estimate of
    % the mean value of the ESF & LSF is off !!!
    %  
    % Why is this not a serious issue ?
    % Zeroing out the mean value of the LSF would result in zero contrast @ DC
    % 
    % What's the solution ? 
    % Infer the SFR @ DC using a first order Taylor series approximation to 
    % the SFR @ the first non-zero frequency. So long as the first non-zero
    % ferquency is close to DC, a first order Taylor series approximation
    % should suffice
    % 
    % SFR(\xi=0) = SFR(\xi=\Delta\xi) - (\Delta\xi) * \frac{\partial}{\partial\xi}SFR(\xi) 
    % SFR(\xi=0) = SFR(\xi=\Delta\xi) - (\Delta\xi) * { SFR(\xi=2\Delta\xi) - SFR(\xi=\Delta\xi) }
    % In the DFT domain,
    % \Delta\xi = 1, so that
    % SFR(\xi=0)  = SFR(\xi=1) - {SFR(\xi=2)-SFR(\xi=1)} 
    %             = 2*SFR(\xi=1) - SFR(\xi=2)
      CFA_struct.(strCh).SFR(1) = ...
          2*CFA_struct.(strCh).SFR(2) - CFA_struct.(strCh).SFR(3);
        
% TODO : propagate change in SFR @ DC, to ESF
    
  % -------------    
  % Normalize SFR
  % -------------
    CFA_struct.(strCh).SFR = CFA_struct.(strCh).SFR/CFA_struct.(strCh).SFR(CFA_struct.(strCh).freq==0);
  end
  
end
  

% -------------------------------------------------------------
% Super-resolve the Edge Spread Function for all color channels
% -------------------------------------------------------------
function superResolveESF(OPERATION,ESF_supersampling_factor,h_deriv)
% Each pixel in the ROI contrubutes to a specific sample within the super-sampled ESF
% The super-sampled ESF is  cell array where each cell contains the list of pixels
% in the ROI that contributed to a specific sample of the 

  global CFA_struct
  
  ESF_inter_sample_spacing = 1/ESF_supersampling_factor;

  % minimum number of samples that cast a vote
  % min_num_samples = ceil(0.5/abs(CFA_struct.slope));
  min_num_samples = 8; % corresponds to a slope of 7.125 degrees
  
  % 
  CFA_img_crop = CFA_struct.Data( CFA_struct.ROI(1,1):CFA_struct.ROI(2,1),...
                                  CFA_struct.ROI(1,2):CFA_struct.ROI(2,2),: ); 
  CFA_img_crop = adjustDR(CFA_img_crop,0.8,0.2,true);
  [nPixelsY,nPixelsX,K] = size(CFA_img_crop);
                  
  for strCh = {'Red','Green','Blue'}
    switch strCh{:}
      case 'Red'
        ROI_Data  = CFA_img_crop(:,:,1);
        [Yt,Xt]   = find( ~isnan(ROI_Data) );
      case 'Green'
        ROI_Data  = CFA_img_crop(:,:,2);
        [Yt,Xt]   = find( ~isnan(ROI_Data) );
      case 'Blue'
        ROI_Data  = CFA_img_crop(:,:,3);
        [Yt,Xt]   = find( ~isnan(ROI_Data) );
    end     
    lind = (Xt(:)-1)*nPixelsY + Yt(:); % convert row,col index to linear index
    strCh = strcat(strCh{:},'_Channel');    

% NOTES
% -----
% The expression (Xt(:)-Yt(:)*CFA_struct.slope-CFA_struct.intercept
% measures the difference in the intercept of the slanted edge and a 
% line parallel to the slanted edge & passing through the pixel (Yt,Xt) 
% The differential intercept is measured w.r.t the 1st column of the image
% The differential intercept needs to cosine projected onto the ESF
% sampling direction to determine the sample position in the ESF grid
  sample = ( (Xt(:)-Yt(:)*CFA_struct.slope-CFA_struct.intercept)/cosd(atand(CFA_struct.slope) ) * 1/ESF_inter_sample_spacing  );
    % sample is a rational number that indicates the sample position on a
    % discrete grid with inter-sample-spacing of "ESF_inter_sample_spacing"
    
% Identify optimal sampling phase for the super-sampled ESF grid   
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Premise : if the ESF has been super-sampled @ the correct phase, 
% then the slope of the LSF on either side of the peak value must be
% identical
    if strcmp(OPERATION,'WT_PERP_DIST')
      AnonfnPtr = @(x,W) sum(x.*W)/sum(W);
    end
    if strcmp(OPERATION,'MEDIAN')
      AnonfnPtr = @(x,W) median(x);
    end
    if strcmp(OPERATION,'MEAN')
      AnonfnPtr = @(x,W) mean(x);
    end
    
    ESF_CellArray = {};
    Wt_CellArray = {};
    sampling_phase = 0:1/50:1;
    opt_cost = -Inf;    
    opt_theta = Inf;
    for theta = sampling_phase
      % round "sample" to nearest integers & add sampling phase
      uniq_sample = unique(sign(sample).*round(abs(sample))) + theta;
      for nn=1:numel(uniq_sample)
        loc = find( abs(sample-uniq_sample(nn)) < 0.5*ESF_inter_sample_spacing );
        ESF_CellArray{nn} = ROI_Data( lind(loc) );
        Wt_CellArray{nn}  = ( 1 - abs(sample(loc)-uniq_sample(nn))/ESF_inter_sample_spacing ).^(1/4);
        % Wt_CellArray{nn}  = 1 - (abs(sample(loc)-uniq_sample(nn))).^(1/4);
      end
      ESF = cellfun( AnonfnPtr,ESF_CellArray,Wt_CellArray );
      LSF = imfilter(ESF(:),h_deriv(:),NaN,'same');
      LSF(isnan(LSF)) = [];
      LSF = LSF/sign(sum(LSF));
      srt_LSF = sort(LSF,'descend');
      cost = atand((srt_LSF(1)-srt_LSF(2))/ESF_inter_sample_spacing*1/CFA_struct.Pixel_Size) + ...
             atand((srt_LSF(1)-srt_LSF(3))/ESF_inter_sample_spacing*1/CFA_struct.Pixel_Size); 
      % cost = srt_LSF(1)-srt_LSF(3);
      if cost > opt_cost
        opt_theta = theta;
        opt_cost = cost;
      end
    end    
    % Use optimal value of sampling phase to estimate suprer-sampled ESF
    uniq_sample = unique(sign(sample).*round(abs(sample))) + opt_theta;    
    for nn=1:numel(uniq_sample)
      loc = find( abs(sample-uniq_sample(nn)) < 0.5*ESF_inter_sample_spacing );
      ESF_CellArray{nn} = ROI_Data( lind(loc) );
      Wt_CellArray{nn}  = ( 1 - abs(sample(loc)-uniq_sample(nn))/ESF_inter_sample_spacing ).^(1/4);
      % Wt_CellArray{nn}  = 1 - (abs(sample(loc)-uniq_sample(nn))).^(1/4);
    end    
    numSamplesInEachCell   = cellfun( @(x) numel(x),ESF_CellArray );
    CFA_struct.(strCh).ESF = cellfun( AnonfnPtr,ESF_CellArray,Wt_CellArray );
    
    % Each entry of the cell-array "ESF_CellArray" stores the intensities 
    % of pixels in the CFA slanted edge image, that contribute to an ESF sample
    % with the ISS "ESF_inter_sample_spacing" & sampling phase "opt_theta"
 
    % Each elementof the array "numSamplesInEachCell" contains the number of 
    % pixels in the CFA image of the slanted edge that contributed to a
    % specific ESF sample
    
    % discard ESF samples with low votes
    if strcmp(strCh,'Green_Channel')
      lead_idx = find( numSamplesInEachCell >= 2*min_num_samples,1,'first' );
      last_idx = find( numSamplesInEachCell >= 2*min_num_samples,1,'last' );
    else
      lead_idx = find( numSamplesInEachCell >= min_num_samples,1,'first' );
      last_idx = find( numSamplesInEachCell >= min_num_samples,1,'last' );      
    end
    
    % ensure that the number of samples in the ESF/LSF is ODD
    if mod(last_idx-lead_idx+1,2)==0 
      last_idx = last_idx - 1;
    end  
    CFA_struct.(strCh).ESF = CFA_struct.(strCh).ESF(lead_idx:last_idx);
    CFA_struct.(strCh).ESF = CFA_struct.(strCh).ESF(:); 
    %
    CFA_struct.(strCh).sample = uniq_sample(lead_idx:last_idx);
    CFA_struct.(strCh).sample = CFA_struct.(strCh).sample(:);    
  end
end

% -----------------------------------------------------------------------
% 2-Pass Edge Spread Function Denoising
% -----------------------------------------------------------------------
% Pass-I  : Decompose ESF as [Left-Tail ,  Transition Region, Right-Tail]
%           where the Left & Right tails are sigmoid functions
% Pass-II : Smooth kinks in the ESF obtained from Pass-I using
%           cubic-splines or gauss-hermite polynomials
% 
% Known Issues : Cubic-spline fitting may produce overshoots & undershoots
%                in the ESF
%                The number of gauss-hermite polynomials needed is 
% ------------------------------------------------------------------------
function [ESF,LSF] = DenoiseESF(ESF,h_deriv,DENOISING_ALGORTIHM,strCh,mode)  

bDisp_only_ESF = true;

% PARAMETERS
% ----------  
  sample = 1:numel(ESF);
  sample = sample(:);
  % Identify Noisy Line Spread Function
  LSF = imfilter(ESF,h_deriv(:),'replicate','same'); 
  LSF = LSF/sign(sum(LSF));
  
if ~strcmp(DENOISING_ALGORTIHM,'NONE')
  % Parametric fitting of sloping sigmoid to ESF tails
  Params = {'mx','mn','s','mu','m'};  
  fun = fittype('(mx-mn)/pi*atan((x-mu)/s) + 0.5*(mx+mn) + m*(x-mu)','indep','x','coeff',Params);
  % mu_init = round(mean([sample(runLength(1)),sample(runLength(1)+runLength(2))])); 
  mu_init = sample(abs(LSF)==max(abs(LSF)));
  linearcomponent_slope_init = 0.; 
  
if strcmp(mode,'VERBOSE')
  % PLOTS
  switch(strCh)
    case 'Red_Channel'
      figure(10001)
    case 'Green_Channel'
      figure(10002)
    case 'Blue_Channel'
      figure(10003)   
  end    
  hfig = gcf();
  if ~bDisp_only_ESF
    subplot(211)
    plot( sample,ESF,'ko','markersize',4 ),hold on
  else
    plot( sample,ESF,'k.','markersize',20 ),hold on    
  end
end

  LSF_LEFT_NOISE_FLOOR  = NaN;
  LSF_RIGHT_NOISE_FLOOR = NaN;  
    
  % Idetify index of peak value in noisy super-sampled LSF
  ell = find(LSF==max(LSF));
  % Identify noise floor in LEFT-TAIL of noisy super-sampled LSF
  uniq_LSF_values = flipud(unique(LSF(1:ell)));
  left_runLength = [0,0];
  for nn = 1:numel(uniq_LSF_values)
    bNoisySamples = (LSF(1:ell) < uniq_LSF_values(nn));
    % run-length vector....string of 1's & 0's
    candidate_runLength = diff([ 0 ; find(bNoisySamples(1:end-1) ~= bNoisySamples(2:end)) ; numel(bNoisySamples) ]);
    if numel(candidate_runLength)==2 && ...
       candidate_runLength(2) > left_runLength(2) && ...  % exactly 2 segments
       candidate_runLength(1) > 2*numel(Params)           % ensure that we have enough points for sigmoid fit
      left_runLength = candidate_runLength;
      LSF_LEFT_NOISE_FLOOR = uniq_LSF_values(nn);
    end
  end
  % Identify noise floor in RIGHT-TAIL of noisy super-sampled LSF
  uniq_LSF_values = flipud(unique(LSF(ell+1:end)));
  right_runLength = [0,0];
  for nn = 1:numel(uniq_LSF_values)
    bNoisySamples = (LSF(ell+1:end) < uniq_LSF_values(nn));
    % run-length vector....string of 1's, 0's & 1's
    candidate_runLength = diff([ 0 ; find(bNoisySamples(1:end-1) ~= bNoisySamples(2:end)) ; numel(bNoisySamples) ]);
    if numel(candidate_runLength)==2 && ...
       candidate_runLength(1) > right_runLength(1) && ... % exactly 2 segments
       candidate_runLength(2) > 2*numel(Params)           % ensure that we have enough points for sigmoid fit 
      right_runLength = candidate_runLength;
      LSF_RIGHT_NOISE_FLOOR = uniq_LSF_values(nn);
    end
  end
  % composite runlength
  runLength = [left_runLength(1),left_runLength(2)+right_runLength(1),right_runLength(2)]; 
  
  if isnan(LSF_LEFT_NOISE_FLOOR) || isnan(LSF_RIGHT_NOISE_FLOOR)
    runLength
    error('Could not denoise the ESF !!')
  end
  
  % ---------------------------------------------------------------------
  % Nonlinear least-squares data fitting using Levenberg-Marquardt method
  % ---------------------------------------------------------------------
  % sensitive to initial condition....figure out value of slope using LS method
  MX = median(ESF(1:runLength(1)));
  MN = median(ESF(end-runLength(3):end));

  %% TODO : How to handle overshoots and undershoots in the ESF ? 
  % ------------------------------------------------
  % Initialize parameters before attempting LM fit
  % ------------------------------------------------
  s_init = mean( (sample-mu_init)./tan( pi*(ESF-0.5*(MX+MN))/(MX-MN) ) );  
  Params_init = [MX,MN,s_init,mu_init,linearcomponent_slope_init];  
  %
  fitoptions('Normalize','on');
  
  ESF_Left_Tail = {};       ESF_Right_Tail = {};
  arr_Left = runLength(1)-find(sample==mu_init):-1;
  arr_Right = 1:runLength(1)+runLength(2)-find(sample==mu_init);
  %
%   arr_Left = arr_Left(1);   arr_Right = arr_Right(end);
  % --------------------------------------------------------
  % Fit sloping sigmoid to left-section of super-sampled ESF
  % --------------------------------------------------------
  for nn = 1:numel(arr_Left)
    end_idx           = find(sample==mu_init) + arr_Left(nn);
    curve             = fit( sample(1:end_idx),ESF(1:end_idx),fun,'startpoint',Params_init,'algorithm', 'Levenberg-Marquardt');
    ESF_Left_Tail{nn} = curve(sample(1:end_idx));
  end
  % ---------------------------------------------------------
  % Fit sloping sigmoid to right-section of super-sampled ESF
  % ---------------------------------------------------------
  for mm = 1:numel(arr_Right)
    strt_idx            = find(sample==mu_init) + arr_Right(mm);
    curve               = fit( sample(strt_idx:end),ESF(strt_idx:end),fun,'startpoint',Params_init,'algorithm', 'Levenberg-Marquardt');
    ESF_Right_Tail{mm}  = curve(sample(strt_idx:end));
  end
  % --------------------------------------------------------
  % A series of denoised ESF's can be obtained by pairing 
  % a candidate ESF left tail with the corresponding ESF
  % right tail
  % Evaluate TV-norm of LSF for each combinations & identify
  % the combination that yields the smoothest LSF
  % --------------------------------------------------------
  mse = zeros(numel(arr_Left),numel(arr_Right));
  for nn = 1:numel(arr_Left)
    for mm = 1:numel(arr_Right)
      end_idx       = find(sample==mu_init) + arr_Left(nn);
      strt_idx      = find(sample==mu_init) + arr_Right(mm);
      denoised_ESF  = [ESF_Left_Tail{nn}; ESF(end_idx+1:strt_idx-1) ; ESF_Right_Tail{mm}];
      % check smoothness of LSF by computing second derivative of LSF
      % or 3rd derivative of ESF
      % mse(nn,mm)    = sum((imfilter(denoised_ESF,[-1,8,-13,0,13,-8,1]'/8,'replicate','same')).^2);
      mse(nn,mm)    = sum((imfilter(denoised_ESF,[-0.5,0,0.5]','replicate','same')).^2);
    end
  end
  [nn_opt,mm_opt] = find(mse==min(mse(:)));
  end_idx_opt   = find(sample==mu_init) + arr_Left(nn_opt);
  strt_idx_opt  = find(sample==mu_init) + arr_Right(mm_opt);
  denoised_ESF  = [ESF_Left_Tail{nn_opt}; ESF(end_idx_opt+1:strt_idx_opt-1) ; ESF_Right_Tail{mm_opt}];
  %
%   numel_ESF_Left_Tail = numel(ESF_Left_Tail{nn_opt});
%   numel_ESF_Right_Tail = numel(ESF_Right_Tail{mm_opt});
%   numel_ESF_Transition = numel(ESF(end_idx_opt+1:strt_idx_opt-1)); 
  %
  ESF = denoised_ESF;
  LSF = imfilter(ESF,h_deriv(:),'replicate','same');
  LSF = LSF/(sum(LSF));
else
% No denoising  
  LSF = imfilter(ESF,h_deriv(:),'replicate','same');
  LSF = LSF/(sum(LSF));
end

if strcmp(mode,'VERBOSE')
  if ~bDisp_only_ESF
    subplot(212)
    plot( sample,LSF,'ko','markersize',4 ),hold on
  else
    switch(strCh)
    case 'Red_Channel'
      figure(20001)
    case 'Green_Channel'
      figure(20002)
    case 'Blue_Channel'
      figure(20003)
    end
    hfig2 = gcf();
    plot( sample,LSF,'k.','markersize',20 ),hold on   
  end
end


if strcmp(DENOISING_ALGORTIHM,'GAUSS_HERMITE')
  LSF_hat = zeros(size(LSF));
  % ------------------------------------------
  % Fit a hermite polynomial to the noisy LSF
  % ------------------------------------------
  MAX_NUM_PASSES = 11; % 1 mainlobe + 4 sidelobes
  % This could also be determined by the number of peaks in the LSF
  for pass = 1:MAX_NUM_PASSES
    err = LSF - LSF_hat;
    mu_init = sample(abs(err)==max(abs(err)));
    [V,loc] = min( abs(err-std(err)) );
    s_init  = abs(loc-mu_init);
    c0_init = GaussHermite_Expansion( sample-mu_init,1,0,0,0,0,s_init )' * err;
    c1_init = GaussHermite_Expansion( sample-mu_init,0,1,0,0,0,s_init )' * err;
    c2_init = GaussHermite_Expansion( sample-mu_init,0,0,1,0,0,s_init )' * err;
    c3_init = GaussHermite_Expansion( sample-mu_init,0,0,0,1,0,s_init )' * err;   
    c4_init = GaussHermite_Expansion( sample-mu_init,0,0,0,0,1,s_init )' * err;   
    Params_init = [c0_init,c1_init,c2_init,c3_init,c4_init,s_init];  
    fun = fittype( 'GaussHermite_Expansion(x,c0,c1,c2,c3,c4,sigma)' );
    fitoptions('Normalize','on');
    fitresult  = fit( sample-mu_init,err,fun,'StartPoint',Params_init,'algorithm','Levenberg-Marquardt');
    LSF_hat    = LSF_hat + fitresult( sample-mu_init ); 
    %
    err = LSF - LSF_hat;
    if sqrt(mean(err.^2)) < 5e-4
      break
    end
  end   
%   pass
  LSF = LSF_hat/(sum(LSF_hat));
end

% ------------------------------------------
% Fit a cubic-spline fit to the denoised ESF
% ------------------------------------------
if strcmp(DENOISING_ALGORTIHM,'CUBIC_SPLINE')
  [cs,p] = csaps(sample,denoised_ESF);
  cs = csaps(sample,denoised_ESF,p);  
  ESF = fnval(cs,sample);
  LSF = imfilter(ESF,h_deriv(:),'replicate','same');
  LSF = LSF/(sum(LSF));
end

if strcmp(mode,'VERBOSE')
  % -------------------
  % Annotated ESF PLOTS
  % -------------------
    if ~bDisp_only_ESF
      subplot(211)
    end
    figure(hfig)
    if ~strcmp(DENOISING_ALGORTIHM,'NONE')
      patch( sample([1,end_idx_opt,end_idx_opt,1]),...
            [min(ESF),min(ESF),max(ESF),max(ESF)],...
            [0.1,0.1,0.1],'edgecolor','none','FaceAlpha',0.35 )
      patch( sample([strt_idx_opt,end,end,strt_idx_opt]),...
            [min(ESF),min(ESF),max(ESF),max(ESF)],...
            [0.6,0.6,0.6],'edgecolor','none','FaceAlpha',0.35 ) 
    end  
    %
    switch(strCh)
      case 'Red_Channel'
        plot( sample,ESF,'r','linewidth',2 )
      case 'Green_Channel'
        plot( sample,ESF,'g','linewidth',2 )
      case 'Blue_Channel'
        plot( sample,ESF,'b','linewidth',2 )
    end
    axis off
    % title(strtok(strCh,'_'),'fontsize',18)
    set(gcf,'Color',[1,1,1])

  % -------------------
  % Annotated LSF PLOTS
  % -------------------
    if ~bDisp_only_ESF
      subplot(212)
    else
      figure(hfig2)
      switch(strCh)
        case 'Red_Channel'
          plot( sample,LSF,'r','linewidth',2 )
        case 'Green_Channel'
          plot( sample,LSF,'g','linewidth',2 )
        case 'Blue_Channel'
          plot( sample,LSF,'b','linewidth',2 )
      end
      axis off
      set(gcf,'Color',[1,1,1])
    end  
end
%  
end

% -----------------------------------------------------------------
% Identify slope & intercept of slanted edge from green CFA samples
% Also identify the maximum & minimum values of the ESF
% -----------------------------------------------------------------
function  findSlopeIntercept_SlantedEdge(mode)
  
  % PARAMETERS
  global CFA_struct
  
  if ~isfield(CFA_struct,'slope')
    % first call to this function...so go ahead & allocate storage for
    % internal data structures
    CFA_struct.slope     = [];
    CFA_struct.intercept     = [];    
    CFA_struct.Red_Channel = ...
      struct('ROI_mxval',[],'ROI_mnval',[],...
             'ESF',[],'LSF',[],'sample',[],'SFR',[],'freq',[]);
    CFA_struct.Green_Channel = ...
      struct('ROI_mxval',[],'ROI_mnval',[],...
             'ESF',[],'LSF',[],'sample',[],'SFR',[],'freq',[]);
    CFA_struct.Blue_Channel = ...
      struct('ROI_mxval',[],'ROI_mnval',[],...
             'ESF',[],'LSF',[],'sample',[],'SFR',[],'freq',[]);       
  end
  %
  
  % Crop CFA image to obtain ROI
  CFA_img_crop = CFA_struct.Data( CFA_struct.ROI(1,1):CFA_struct.ROI(2,1),...
                                  CFA_struct.ROI(1,2):CFA_struct.ROI(2,2),: ); 
  %
  ROI_mxval = max(CFA_img_crop,[],3);
  ROI_mnval = max(CFA_img_crop,[],3);
  %
  CFA_struct.Red_Channel.ROI_mxval    = ROI_mxval(1);
  CFA_struct.Red_Channel.ROI_mnval    = ROI_mnval(1);
  CFA_struct.Green_Channel.ROI_mxval  = ROI_mxval(2);
  CFA_struct.Green_Channel.ROI_mnval  = ROI_mnval(2);
  CFA_struct.Blue_Channel.ROI_mxval   = ROI_mxval(3);
  CFA_struct.Blue_Channel.ROI_mnval   = ROI_mxval(3);
  
  % Adjust Dynamic Range of ROI by resetting black point & white point
  % disp('Max & Min values in ROI')
  % [max(CFA_img_crop(:)),min(CFA_img_crop(:))]
  fprintf( 'Contrast of Slanted Edge in ROI %2.4f\n',max(CFA_img_crop(:))/min(CFA_img_crop(:)) );
  %
  CFA_img_crop = adjustDR(CFA_img_crop,0.8,0.2,true);
  %
  
  % --------------------------
  % determine edge orientation
  % --------------------------
  Green_img_crop = CFA_img_crop(:,:,2);
  Green_img_crop(isnan(Green_img_crop)) = 0.;
  med1 = median(Green_img_crop,1);
  med2 = median(Green_img_crop,2);
  clear Green_img_crop
  if std(med1) < std(med2)
    % edge subtends small angle w.r.t horizontal
    disp('Slanted edge subtends small angle w.r.t horizontal !!')
    CFA_struct.Data = permute(CFA_struct.Data,[2,1,3]); % transpose image of each color channel  
    CFA_struct.ROI  = fliplr(CFA_struct.ROI);
    CFA_img_crop    = permute(CFA_img_crop,[2,1,3]);
  else
    % edge subtends small angle w.r.t vertical
    disp('Slanted edge subtends small angle w.r.t vertical !!')
  end
  [nPixelsY,nPixelsX,K] = size(CFA_img_crop);
  
  %% Identify coordinates of points that make up the slanted edge
  %% provides sub-pixel precision
  img_buf  = sum( max(CFA_img_crop,0),3 ); % convert NaN's in CFA_img_crop to 0's before summing up all channels
  
  % Compute gradient along columns
  % The following assumes that the edge orientation is vertical
  edge_buf = abs( imfilter(img_buf,[0,0,0;1,0,-1;0,0,0],NaN,'same') ) ;
  %     
  % Identify location of edge in CFA image....CFA edge detection
  Ye = 1:nPixelsY;
  [V1,Xe1] = max( edge_buf,[],2 ); 
  % Discard first & last entries as edge filtering produces unreliable values
  Xe1 = Xe1(2:end-1); Ye  = Ye(2:end-1);
  Xe = transpose(Xe1);  
  %
  
%   figure
%   imshow( adjustDR(edge_buf,1,0,'true') )
%   hold on
%   plot( Xe,Ye,'y.','markersize',6 )
%   error('done')

  
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if strcmp(mode,'VERBOSE') 
  figure(4), cla
  imshow( CFA_img_crop ),
  hold on
  plot( Xe,Ye,'y.','markersize',6 ) % Raw data
end
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

%% Robust Line-fit implementation (Brute-Force approach)  
  % Fit lines to each combination of 2 points drawn from the point cloud
  C = nchoosek(1:numel(Ye),2);  
  Pt1 = struct( 'X',Xe(C(:,1)),'Y',Ye(C(:,1)) );
  Pt2 = struct( 'X',Xe(C(:,2)),'Y',Ye(C(:,2)) );
  % line equation x = m y + c
  Candidate_slope = (Pt2.X-Pt1.X)./(Pt2.Y-Pt1.Y);  % slope
  Candidate_intercept = Pt2.X - Candidate_slope .* Pt2.Y;  % intercept
  % For each candidate slope & intercept find the perpendicular distance
  % of all points in the point cloud
  % A point is deemed an inlier if its perpendicular distance to the
  % candidate line is smaller than 2 pixels
  ind_Inliers = [];
  MX_Num_Inliers = -Inf;
  MN_residual = Inf;
%% TODO : Vectorize FOR loop using blocks
  for nn = 1:numel(Candidate_slope)
    % for line x = my + c
    % the perpendicular distance is given by abs(x_i-m*y_i-c)/sqrt(1+m^2)
    Perp_Distance = abs(Xe - Candidate_slope(nn)*Ye - Candidate_intercept(nn))./sqrt( 1+Candidate_slope(nn)^2 );
    %
    D = 2;
    %
    if sum((Perp_Distance < D)) > MX_Num_Inliers
      %
      ind_Inliers = find(Perp_Distance < D);
      MN_residual = sum(Perp_Distance(ind_Inliers));
      MX_Num_Inliers = sum((Perp_Distance < D));
      %
    end
    if sum((Perp_Distance < D)) == MX_Num_Inliers && ...
       sum(Perp_Distance(Perp_Distance < D)) < MN_residual 
      %
      ind_Inliers = find(Perp_Distance < D);
      MN_residual = sum(Perp_Distance(ind_Inliers));
      MX_Num_Inliers = sum((Perp_Distance < D));
      %
    end
  end 
  Ye = Ye(ind_Inliers);
  Xe = Xe(ind_Inliers);
  
  % --------------------------------------
  % Parametric Model for linear regression 
  % --------------------------------------
  mu_Xe = mean(Xe);
  mu_Ye = mean(Ye);
  CFA_struct.slope = sum((Ye-mu_Ye).*(Xe-mu_Xe))/sum((Ye-mu_Ye).^2);
  CFA_struct.intercept = mu_Xe - CFA_struct.slope*mu_Ye; 
  %
  fprintf('Slant angle of edge %2.4f degrees\n',atand(CFA_struct.slope));
  % fprintf('Perpendicular distance of worst offender %3.4f pixels\n',max(Perp_distance));

  if abs(atand(CFA_struct.slope)) < 3.5 || abs(atand(CFA_struct.slope)) > 7
    fprintf('WARNING: SFR measurement is prone to severe inaccuracies due to poor slant angle !!')
  end
  
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if strcmp(mode,'VERBOSE')  
  figure( 4 )
  plot( (Ye*CFA_struct.slope+CFA_struct.intercept),Ye,'r.' ) % Line fit
  title('LS estimate of slanted edge overlaid on CFA Green channel')  
end
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

end

% ------------------------------------------------------------------------------
% Compensate for SFR contrast loss due to finite difference derivative filtering
% 
% ------------------------------------------------------------------------------
function [SFR,freq] = Compensate_4_DerivativeFiltering(SFR,SFR_starts_at_DC,h_deriv)
  len = numel(h_deriv);
 
  if SFR_starts_at_DC
    numel_SFR = 2*numel(SFR)-1;
    freq = -floor(numel_SFR/2):floor(numel_SFR/2);
    freq = freq/numel_SFR;
    freq = freq(round(numel_SFR/2):end);
  else
    freq = -floor(numel(SFR)/2):floor(numel(SFR)/2);
    freq = freq/numel(SFR);
  end
  % freq assumes values in the interval (-0.5,0.5)
  % The normalized spatial frequency in units of cyc/pix assumes a value of
  % 0.5 for the Nyquist frequency 
  COMP = zeros(size(freq));
  for k = 1:floor(len/2)
    COMP = COMP + 2 * h_deriv(round(len/2)+k) * sin(2*pi*k*freq); 
  end
  COMP = abs(COMP)./abs(2*pi*freq);
  COMP(freq==0) = 1.;
  
  %   % Peter Burns clips the value of 1/sinc to 10...as the argument of sinc
  %   % approaches 0.99 1/sinc exceeds 100, sinc(1) is a huge number !!!
  %   N = 2;                      
  %   GREEN_SFR(N:end-N) = GREEN_SFR(N:end-N) ./ (sinc(freq(N:end-N)/max(freq(:))));
  %   GREEN_SFR(1:N-1) = GREEN_SFR(N);
  %   GREEN_SFR(end-N+1:end) = GREEN_SFR(end-N);

  SFR = SFR(:);
  COMP = COMP(:);
  COMP = min(COMP,0.1);
  
  N = 1;                      
  if SFR_starts_at_DC
    SFR(1:end-N) = SFR(1:end-N) ./ COMP(1:end-N);
    SFR(end-N+1:end) = SFR(end-N);
  else
    SFR(N+1:end-N) = SFR(N+1:end-N) ./ COMP(N:end-N);
    SFR(1:N) = SFR(N+1);
    SFR(end-N+1:end) = SFR(end-N);    
  end
end

function [img,mx_old,mn_old] = adjustDR( img,mx,mn,bEnableAdjustments )
  if bEnableAdjustments
    mx_old = max(img(:));
    mn_old = min(img(:));    
    img(:) = (img(:)-mn_old)/(mx_old-mn_old)*(mx-mn) + mn;
  else  
    img = img;
  end  
end
