function driver_CFASFR()
  % example code to test the CFASFR algorithm
  
  clc,close all,
  clear all

% allocate storage for global variables
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  global CFA_struct;

% Provide filename
% ~~~~~~~~~~~~~~~~
  fileName  = 'ISO16067_Chart.CR2';

% Uncomment lines 17-20 to identify ROI
% When done, comment lines 17-20
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%   sRGB_Img = CFA_imread( fileName,'sRGB' );
%   figure
%   imshow( sRGB_Img ), axis on
%   error('done')

  CFA_Img  = CFA_imread( fileName,'Linear' );
    
% Identify ROI containing slanted edge in CFA image
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  % vertical edge in ROI...size 181 rows x 61 cols
  CropTopLeft = [2773-30,2983-90]; % row,col
  CropBottomRight = [2773+30,2983+90]; % row,col  
  
% Identify detector pixel pitch
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  PixSize = 4.17e-3; % expressed in mm
  
% The SFR_CFA_SlantedEdge(...) routine fills up the following structure
  CFA_struct  = struct('Data',CFA_Img,...
                       'Pixel_Size',PixSize,...
                       'ROI',[CropTopLeft(1),CropTopLeft(2) ; CropBottomRight(1),CropBottomRight(2)]);
% User-defined parameters for SFR estimation
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ESF_supersampling_factor  = 2;                  % integers between 1 & 4 
  DENOISING_ALGORITHM       = 'GAUSS_HERMITE';    % options are CUBIC_SPLINE,GAUSS_HERMITE,NONE (DEFAULT: GAUSS_HERMITE)
  OPERATION                 = 'WT_PERP_DIST';     % options are MEAN/MEDIAN/WT_PERP_DIST (DEFAULT: WT_PERP_DIST)       
  strMode                   = 'SUMMARY';          % options are SUMMARY/VERBOSE (DEFAULT: SUMMARY)
  SFR_CFA_SlantedEdge( ESF_supersampling_factor, OPERATION, DENOISING_ALGORITHM,strMode );  
  
% ~~~~~
% PLOTS
% ~~~~~
  figure,
  plot( CFA_struct.Red_Channel.freq,  CFA_struct.Red_Channel.SFR,'r-','linewidth',2 )
  hold on  
  plot( CFA_struct.Green_Channel.freq,CFA_struct.Green_Channel.SFR,'g-','linewidth',2 )
  plot( CFA_struct.Blue_Channel.freq, CFA_struct.Blue_Channel.SFR,'b-','linewidth',2 )  
  %
  plot( 1/(2*PixSize)*[1,1],[0,1],'color',[0.65,0.65,0.65],'linewidth',1 )
  str = sprintf('Detector Nyquist Frequency %4.4f (cyc/mm)',1/(2*PixSize));
  text( 0.975/(2*PixSize),0.25,str,'rotation',90,'interpreter','latex','fontsize',15 );
  %
  axis tight,grid on
  xlabel('Spatial Frequency (cyc/mm)','interpreter','latex','fontsize',15)
  ylabel('Normalized Spatial Frequency Response','interpreter','latex','fontsize',15)
  set(gcf,'units','normalized','outerposition',[0 0 1 1],'PaperPositionMode','auto')  
  
end